Unit ClassServices;

Interface

Uses WinSvc, Sorting,
     ComCtrls;

Const
      SLUZBY_SLOUPEC_DISPLAYNAME = 0;
      SLUZBY_SLOUPEC_NAME        = 1;
      SLUZBY_SLOUPEC_TYPE        = 2;
      SLUZBY_SLOUPEC_STATUS      = 3;
      SLUZBY_MAX_SLOUPCU         = 4;

Type
     TServiceRecord = Record
                      Info : _ENUM_SERVICE_STATUS;
                      ServiceName : String;
                      ServiceDisplayName : String;
                      end;
     PServiceRecord = ^TServiceRecord;
     TSluzbySortStyle = (sssDisplayName, sssName, sssType, sssStatus);

     TSluzby = Class (TSortList)
               Private
                 FSortStyle : TSluzbySortStyle;
               Protected
                 Procedure SetSortStyle(H:TSluzbySortStyle);               Virtual;
                 Function  JeVetsi(A,B:Pointer; K:Cardinal):Boolean; Override;
                 Function  JeRovno(A,B:Pointer; K:Cardinal):Boolean; Override;
               Public
                Constructor Create(AListView:TListView); Override;
                Destructor Destroy; Override;
                Procedure Enumerate; Virtual;
                Procedure   ListViewVykresliJednoho(Sender:TObject; Item:TListItem);   Override;
                Procedure   ListViewColumnClick(Sender: TObject; Column: TListColumn); Override;

                Function  GetService(Index:Integer):PServiceRecord;
                Property SortStyle : TSluzbySortStyle Read FSortStyle Write SetSortStyle;
               end;

Implementation

Uses Windows, SysUtils, Forms;

Const
      KRITERIUM_NAME = 0;
      KRITERIUM_DISPLAY_NAME = 1;
      KRITERIUM_TYPE = 2;
      KRITERIUM_STATUS = 3;


Function ServiceTypeToString(S:Cardinal):String;
begin
Result:='';
If (S And SERVICE_INTERACTIVE_PROCESS > 0) Then
   begin
   Result:=' (Interaktivn)';
   S:=S And Not SERVICE_INTERACTIVE_PROCESS;
   end;
Case S Of
     SERVICE_WIN32_OWN_PROCESS   : Result:='Vlastn proces'+Result;
     SERVICE_WIN32_SHARE_PROCESS : Result:='Sdlen proces'+Result;
     end;
end;

Function ServiceStatusToString(S:Cardinal):String;
begin
Result:='';
Case S Of
     SERVICE_STOPPED          : Result:='Neaktivn';
     SERVICE_START_PENDING    : Result:='Ped startem';
     SERVICE_STOP_PENDING     : Result:='Ped ukonenm';
     SERVICE_RUNNING          : Result:='B';
     SERVICE_CONTINUE_PENDING : Result:='Ped odpauzovnm';
     SERVICE_PAUSE_PENDING    : Result:='Pozastavuje se';
     SERVICE_PAUSED           : Result:='Pozastavena';
     end;
end;

// TSluzby

Constructor TSluzby.Create(AListView:TListView);
begin
Inherited Create(AListView);
Kriteria.Add(Pointer(KRITERIUM_DISPLAY_NAME));
FSortStyle:=sssName;
Opacne:=False;
PocetSloupcu:=SLUZBY_MAX_SLOUPCU;
FJmena.LoadFromFile(ExtractFilePath(Application.ExeName)+'Data\service.col');
end;

Destructor TSluzby.Destroy;
begin
Clear;
Kriteria.Free;
Inherited Destroy;
end;

Function TSluzby.GetService(Index:Integer):PServiceRecord;
begin
Result:= Items[Index];
end;

Procedure TSluzby.SetSortStyle(H:TSluzbySortStyle);
begin
If FSortStyle = H Then
   begin
   Opacne:= Not Opacne;
   Serad;
   Exit;
   end
Else begin
     FSortStyle:=H;
     Opacne:=False;
     Kriteria.Clear;
     Kriteria.Add(Pointer(KRITERIUM_DISPLAY_NAME));
     Case FSortStyle Of
          sssName : Kriteria.Insert(0,Pointer(KRITERIUM_NAME));
          sssDisplayName : Kriteria.Insert(0,Pointer(KRITERIUM_DISPLAY_NAME));
          sssType : Kriteria.Insert(0,Pointer(KRITERIUM_TYPE));
          sssStatus : Kriteria.Insert(0,Pointer(KRITERIUM_STATUS));
          end;
     end;
Serad;
end;

Function  TSluzby.JeVetsi(A,B:Pointer; K:Cardinal):Boolean;
Var X, Y : PServiceRecord;
begin
X:=A;
Y:=B;
Result:=False;
Case K Of
     KRITERIUM_NAME : Result:= AnsiStrComp(PAnsiChar(X.ServiceName),PAnsiChar(Y.ServiceName)) > 0;
     KRITERIUM_DISPLAY_NAME : Result:= AnsiStrComp(PAnsiChar(X.ServiceDisplayName),PAnsiChar(Y.ServiceDisplayName)) > 0;
     KRITERIUM_TYPE : Result:= X.Info.ServiceStatus.dwServiceType > Y.Info.ServiceStatus.dwServiceType;
     KRITERIUM_STATUS : Result:= X.Info.ServiceStatus.dwCurrentState > Y.Info.ServiceStatus.dwCurrentState;
     end;
end;

Function  TSluzby.JeRovno(A,B:Pointer; K:Cardinal):Boolean;
Var X, Y : PServiceRecord;
begin
X:=A;
Y:=B;
Result:=False;
Case K Of
     KRITERIUM_NAME : Result:= AnsiStrComp(PAnsiChar(X.ServiceName),PAnsiChar(Y.ServiceName)) = 0;
     KRITERIUM_DISPLAY_NAME : Result:= AnsiStrComp(PAnsiChar(X.ServiceDisplayName),PAnsiChar(Y.ServiceDisplayName)) = 0;
     KRITERIUM_TYPE : Result:= X.Info.ServiceStatus.dwServiceType = Y.Info.ServiceStatus.dwServiceType;
     KRITERIUM_STATUS : Result:= X.Info.ServiceStatus.dwCurrentState = Y.Info.ServiceStatus.dwCurrentState;
     end;
end;

Procedure TSluzby.Enumerate;
Const MAX_SERVICES = $3FF;
Type
    TPoleSluzeb = Array [0..MAX_SERVICES-1] Of _ENUM_SERVICE_STATUS;
    PPoleSluzeb = ^TPoleSluzeb;
    TEnumServiceStatus = Function (hSCManager:SC_HANDLE; dwServiceType,dwServiceState:DWORD; Buffer:PPoleSluzeb; cbBufSize:DWORD; Var pcbBytesNeeded,lpServicesReturned,lpResumeHandle:DWORD):Bool; StdCall;
Var hManager : SC_HANDLE;
    hAdvAPI32 : THandle;
    BytesNeeded : DWORD;
    SCount      : DWORD;
    Check       : DWORD;
    EnumServicesStatus : TEnumServiceStatus;
    PoleSluzeb : PPoleSluzeb;
    I : Integer;
    X : PServiceRecord;
begin
Clear;
hAdvAPI32:=LoadLibrary('advapi32.dll');
If hAdvAPI32 = 0 Then
   Exit;
EnumServicesStatus:=GetProcAddress(hAdvAPI32,'EnumServicesStatusA');
If @EnumServicesStatus = Nil Then
   begin
   FreeLibrary(hAdvAPI32);
   Exit;
   end;
hManager:=OpenSCManager(Nil,Nil,SC_MANAGER_ENUMERATE_SERVICE);
If hManager > 0 Then
   begin
   GetMem(PoleSluzeb,65536);
   BytesNeeded:=MAX_SERVICES*SizeOf(_ENUM_SERVICE_STATUS);;
   SCount:=MAX_SERVICES;
   Check:=0;
   If EnumServicesStatus(hManager,SERVICE_WIN32,SERVICE_STATE_ALL,PoleSluzeb,65536,BytesNeeded,SCount,Check) Then
      begin
      For I:=0 To SCount-1 Do
          begin
          New(X);
          X.Info:=PoleSluzeb[I];
          X.ServiceName:=String(X.Info.lpServiceName);
          X.ServiceDisplayName:=String(X.Info.lpDisplayName);
          X.Info.lpServiceName:=Nil;
          X.Info.lpDisplayName:=Nil;
          Add(X);
          end;
      end;
   FreeMem(PoleSluzeb,65536);
   CloseServiceHandle(hManager);
   end;
FreeLibrary(hAdvAPI32);
Serad;
If Assigned(FListView) Then
   begin
   FListView.Items.Count:=Count;
   FListview.Invalidate;
   end;
end;

Procedure TSluzby.ListViewColumnClick(Sender: TObject; Column: TListColumn);
begin
Enumerate;
SortStyle:=TSluzbySortStyle(Column.Tag);
end;

Procedure TSluzby.ListViewVykresliJednoho(Sender:TObject; Item:TListItem);
Var S : PServiceRecord;
begin
S:=GetService(Item.Index);
With Item Do
     begin
     If SloupecDejStav(SLUZBY_SLOUPEC_DISPLAYNAME) Then
        Caption:=S.ServiceDisplayName;
     If SloupecDejStav(SLUZBY_SLOUPEC_NAME) Then
        SubItems.Add(S.ServiceName);
     If SloupecDejStav(SLUZBY_SLOUPEC_TYPE) Then
        SubItems.Add(ServiceTypeToString(S.Info.ServiceStatus.dwServiceType));
     If SloupecDejStav(SLUZBY_SLOUPEC_STATUS) Then
        SubItems.Add(ServiceStatusToString(S.Info.ServiceStatus.dwCurrentState));
     end;
end;

End.
