Unit Debugging;

Interface

Uses Windows;

Type
    TDebugProcedure = Function (Event: _DEBUG_EVENT):Boolean; StdCall;

Function  StartDebuggingProcess(PID:Cardinal; ADebugProc:TDebugProcedure):Boolean; StdCall;
Function  StopDebuggingProcess(PID:Cardinal):Boolean; StdCall;

Implementation

Uses OSVersion,
     Classes,
     SyncObjs;

Type
    T2DPole = Array [0..2] Of Cardinal;
    P2DPole = ^T2DPole;

Var StopDebugPID : Cardinal;
    A : T2DPole;
    ThreadList : TList;
    C : TCriticalSection;

Function DebuggerMain(X:P2DPole):Boolean; StdCall;          Forward;
Function _DebugActiveProcessStop(PID:Cardinal):Boolean;     Forward;
Function _DebugSetProcessKillOnExit(Kill:Boolean):Boolean;  Forward;

Function  StartDebuggingProcess(PID:Cardinal; ADebugProc:TDebugProcedure):Boolean; StdCall;
Var TID : Cardinal;
    hThread : THandle;
begin
C.Enter;
A[0]:=PID;
A[1]:=Cardinal(@ADebugProc);
hThread:=CreateThread(Nil,0,@DebuggerMain,@A,CREATE_SUSPENDED,TID);
A[2]:=hThread;
ThreadList.Add(Pointer(hThread));
ResumeThread(hThread);
C.Leave;
end;

Function DebuggerMain(X:P2DPole):Boolean; StdCall;
Var D : _DEBUG_EVENT;
    Pokracuj : Boolean;
    PID : Cardinal;
    ADebugProc : TDebugProcedure;
    I : Integer;
    hThread : Cardinal;
begin
C.Enter;
PID:=A[0];
ADebugProc:=TDebugProcedure(A[1]);
hThread:=A[2];
C.Leave;
Result:=DebugActiveProcess(PID);
If (WinXP) And (Result) Then
   begin
   Result:=_DebugSetProcessKillOnExit(False);
   Pokracuj:=True;
   While Pokracuj Do
         begin
         If WaitForDebugEvent(D,1000) Then
            begin
            Pokracuj:=ADebugProc(D);
            ContinueDebugEvent(D.dwProcessId,D.dwThreadId,DBG_CONTINUE);
            end;
         C.Enter;
         If StopDebugPID = PID Then
            begin
            Pokracuj:=False;
            StopDebugPID:=0;
            end;
         C.Leave;
         end;
   Result:=_DebugActiveProcessStop(PID);
   end;
C.Enter;
For I:=0 To ThreadList.Count-1 Do
    If Cardinal(ThreadList[I]) = hThread Then
       begin
       CloseHandle(hThread);
       ThreadList.Delete(I);
       Break;
       end;
C.Leave;
end;

Function  StopDebuggingProcess(PID:Cardinal):Boolean; StdCall;
Var I : Integer;
begin
If WinXP Then
   begin
   C.Enter;
   For I:=0 To ThreadList.Count-1 Do
       SuspendThread(Cardinal(ThreadList[I]));
   StopDebugPID:=PID;
   Result:=True;
   For I:=0 To ThreadList.Count-1 Do
       ResumeThread(Cardinal(ThreadList[I]));
   C.Leave;
   end
Else Result:=False;
end;

Function _DebugActiveProcessStop(PID:Cardinal):Boolean;
Var Proc : Function (dwPID:Cardinal):Boolean; StdCall;
begin
Result:=False;
Proc:=GetProcAddress(GetModuleHandle('kernel32.dll'),'DebugActiveProcessStop');
If Assigned(Proc) Then
   Result:=Proc(PID);
end;

Function _DebugSetProcessKillOnExit(Kill:Boolean):Boolean;
Var Proc : Function (a:Boolean):Boolean; StdCall;
begin
Result:=False;
Proc:=GetProcAddress(GetModuleHandle('kernel32.dll'),'DebugSetProcessKillOnExit');
If Assigned(Proc) Then
   Result:=Proc(Kill);
end;

Initialization
StopDebugPID:=0;
ThreadList:=TList.Create;
C:=TCriticalSection.Create;

Finalization
ThreadList.Free;
C.Free;
End.
